package Fennec::Test;
use strict;
use warnings;

1;

__END__

=pod

=head1 NAME

Fennec::Test - Base class for Fennec tests.

=head1 DESCRIPTION

Base class for Fennec tests. Currently this is an empty base class, it is
simply useful for checking C<$Thing-E<gt>isa( 'Fennec::Test' );>

=head1 AUTHORS

Chad Granum L<exodist7@gmail.com>

=head1 COPYRIGHT

Copyright (C) 2013 Chad Granum

Fennec is free software; Standard perl license.

Fennec is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the license for more details.

=cut
